@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 /*
-  Copyright 2017 Martin Koller, kollix@aon.at
+  Copyright 2017 - 2022 Martin Koller, kollix@aon.at
 
   This file is part of liquidshell.
 
@@ -99,17 +99,19 @@ QStringList NotificationServer::GetCapabilities()
 
 void NotificationServer::CloseNotification(uint id)
 {
+  //qDebug() << "CloseNotification" << id;
   notificationList->closeItem(id);
+  emit NotificationClosed(id, CloseReason::Closed);
 }
 
 //--------------------------------------------------------------------------------
 
 QString NotificationServer::GetServerInformation(QString &vendor, QString &version, QString &spec_version)
 {
-  vendor = "kollix";
+  vendor = "KDE";
   version = "1.0";
   spec_version = "1.2";
-  return vendor;
+  return "liquidshell";
 }
 
 //--------------------------------------------------------------------------------
