@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 /*
-  Copyright 2017 - 2020 Martin Koller, kollix@aon.at
+  Copyright 2017 - 2021 Martin Koller, kollix@aon.at
 
   This file is part of liquidshell.
 
@@ -34,6 +34,7 @@
 #include <QStyle>
 #include <QApplication>
 #include <QScreen>
+#include <QMessageBox>
 #include <QDebug>
 
 #include <KLocalizedString>
@@ -129,7 +130,7 @@ void PkUpdateListItem::getUpdateDetails()
     return;
   }
 
-  //qDebug() << "getUpdateDetails" << package.id;
+  //qDebug() << "getUpdateDetails" << package.id << PackageKit::Daemon::packageName(package.id);
   PackageKit::Transaction *transaction = PackageKit::Daemon::getUpdateDetail(package.id);
   detailsLabel->setText(i18n("Getting details ..."));
   detailsLabel->show();
@@ -248,13 +249,20 @@ PkUpdateList::PkUpdateList(QWidget *parent)
 
 QSize PkUpdateList::sizeHint() const
 {
-  QSize s = scrollArea->widget()->sizeHint() + QSize(2 * scrollArea->frameWidth(), 2 * scrollArea->frameWidth());
-  s.setWidth(s.width() + scrollArea->verticalScrollBar()->sizeHint().width());
-  s.setHeight(layout()->contentsMargins().top() + installButton->sizeHint().height() +
-              ((layout()->spacing() == -1) ? style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing) : layout()->spacing()) +
-              s.height() + layout()->contentsMargins().bottom());
+  QSize s;
 
-  s = s.expandedTo(QSize(500, 300));
+  if ( savedSize.isValid() )
+    s = savedSize;
+  else
+  {
+    s = scrollArea->widget()->sizeHint() + QSize(2 * scrollArea->frameWidth(), 2 * scrollArea->frameWidth());
+    s.setWidth(s.width() + scrollArea->verticalScrollBar()->sizeHint().width());
+    s.setHeight(layout()->contentsMargins().top() + installButton->sizeHint().height() +
+                ((layout()->spacing() == -1) ? style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing) : layout()->spacing()) +
+                s.height() + layout()->contentsMargins().bottom());
+
+    s = s.expandedTo(QSize(500, 300));
+  }
 
   // Qt doc: The size of top-level widgets are constrained to 2/3 of the desktop's height and width.
   QSize screen = DesktopWidget::availableSize();
@@ -265,6 +273,18 @@ QSize PkUpdateList::sizeHint() const
 
 //--------------------------------------------------------------------------------
 
+void PkUpdateList::hideEvent(QHideEvent *event)
+{
+  Q_UNUSED(event)
+
+  if ( itemsLayout->count() )
+    savedSize = size();
+  else
+    savedSize = QSize();
+}
+
+//--------------------------------------------------------------------------------
+
 void PkUpdateList::setRefreshProgress(int progress)
 {
   if ( progress >= 100 )
@@ -294,6 +314,13 @@ void PkUpdateList::setRefreshProgress(int progress)
 
 //--------------------------------------------------------------------------------
 
+bool comparePackageData(const PkUpdates::PackageData &a, const PkUpdates::PackageData &b)
+{
+  return a.summary.localeAwareCompare(b.summary) < 0;
+}
+
+//--------------------------------------------------------------------------------
+
 void PkUpdateList::setPackages(const PkUpdates::PackageList &packages)
 {
   itemsLayout->parentWidget()->layout()->setEnabled(false);
@@ -309,6 +336,7 @@ void PkUpdateList::setPackages(const PkUpdates::PackageList &packages)
   checkAllBox->setChecked(false);
 
   QList<PkUpdates::PackageData> list = packages.values(PackageKit::Transaction::InfoSecurity);
+  std::sort(list.begin(), list.end(), comparePackageData);
   for (const PkUpdates::PackageData &data : list)
   {
     auto *item = new PkUpdateListItem(itemsLayout->parentWidget(), PackageKit::Transaction::InfoSecurity, data);
@@ -318,6 +346,7 @@ void PkUpdateList::setPackages(const PkUpdates::PackageList &packages)
   }
 
   list = packages.values(PackageKit::Transaction::InfoImportant);
+  std::sort(list.begin(), list.end(), comparePackageData);
   for (const PkUpdates::PackageData &data : list)
   {
     auto *item = new PkUpdateListItem(itemsLayout->parentWidget(), PackageKit::Transaction::InfoImportant, data);
@@ -327,6 +356,7 @@ void PkUpdateList::setPackages(const PkUpdates::PackageList &packages)
   }
 
   list = packages.values(PackageKit::Transaction::InfoBugfix);
+  std::sort(list.begin(), list.end(), comparePackageData);
   for (const PkUpdates::PackageData &data : list)
   {
     auto *item = new PkUpdateListItem(itemsLayout->parentWidget(), PackageKit::Transaction::InfoBugfix, data);
@@ -336,18 +366,25 @@ void PkUpdateList::setPackages(const PkUpdates::PackageList &packages)
   }
 
   // all the others
+  list.clear();
   for (PkUpdates::PackageList::const_iterator it = packages.constBegin(); it != packages.constEnd(); ++it)
   {
     if ( (it.key() != PackageKit::Transaction::InfoSecurity) &&
          (it.key() != PackageKit::Transaction::InfoImportant) &&
          (it.key() != PackageKit::Transaction::InfoBugfix) )
     {
-      auto *item = new PkUpdateListItem(itemsLayout->parentWidget(), it.key(), it.value());
-      connect(item, &PkUpdateListItem::toggled, this, &PkUpdateList::countChecked);
-      itemsLayout->addWidget(item);
-      item->show();
+      list.append(it.value());
     }
   }
+  std::sort(list.begin(), list.end(), comparePackageData);
+  for (const PkUpdates::PackageData &data : list)
+  {
+    auto *item = new PkUpdateListItem(itemsLayout->parentWidget(), PackageKit::Transaction::InfoUnknown, data);
+    connect(item, &PkUpdateListItem::toggled, this, &PkUpdateList::countChecked);
+    itemsLayout->addWidget(item);
+    item->show();
+  }
+
   itemsLayout->setEnabled(true);
 
   filterChanged(filterEdit->text());
@@ -557,6 +594,12 @@ void PkUpdateList::installOne()
   packageNoLongerAvailable = false;
   //qDebug() << "installing" << item->package.id;
 
+  connect(transaction.data(), &PackageKit::Transaction::allowCancelChanged, this,
+          [this]()
+          {
+            installButton->setEnabled(transaction->allowCancel());
+          });
+
   connect(transaction.data(), &PackageKit::Transaction::statusChanged, this,
           [item, this]()
           {
@@ -616,6 +659,45 @@ void PkUpdateList::installOne()
               restart = type;
           });
 
+  connect(transaction.data(), &PackageKit::Transaction::eulaRequired, this,
+          [this](const QString &eulaID, const QString &packageID, const QString &vendor, const QString &licenseAgreement)
+          {
+            if ( QMessageBox::question(this, i18n("Accept EULA"),
+                                       QString("%1\n%2\n%3")
+                                               .arg(packageID)
+                                               .arg(vendor)
+                                               .arg(licenseAgreement)) == QMessageBox::Yes )
+              PackageKit::Daemon::acceptEula(eulaID);
+            else
+              installQ.dequeue();
+
+            installOne();
+          });
+
+  connect(transaction.data(), &PackageKit::Transaction::repoSignatureRequired, this,
+          [this](const QString &packageID,
+                 const QString &repoName,
+                 const QString &keyUrl,
+                 const QString &keyUserid,
+                 const QString &keyId,
+                 const QString &keyFingerprint,
+                 const QString &keyTimestamp,
+                 PackageKit::Transaction::SigType type)
+          {
+            if ( QMessageBox::question(this, i18n("Validate Signature"),
+                                       QString("%1\n%2\n%3\n%4\n%5\n%6\n%7")
+                                               .arg(packageID)
+                                               .arg(repoName)
+                                               .arg(keyUrl)
+                                               .arg(keyUserid)
+                                               .arg(keyId)
+                                               .arg(keyFingerprint)
+                                               .arg(keyTimestamp)) == QMessageBox::Yes )
+            {
+              PackageKit::Daemon::installSignature(type, keyId, packageID);
+            }
+          });
+
   connect(transaction.data(), &PackageKit::Transaction::errorCode, this,
           [item, this](PackageKit::Transaction::Error error, const QString &details)
           {
